forge-from-scratch: Example Showing How Forge Can Generate an Application From Scratch
======================================================================================
Author: Lincoln Baxter  
Level: Intermediate  
Technologies: Forge  
Summary: Demonstrates how to generate a fully Java EE compliant project using nothing but JBoss Forge  
Target Product: EAP  
Product Versions: EAP 6.1, EAP 6.2, Forge 1.0.0.Final  
Source: <https://github.com/jboss-developer/jboss-eap-quickstarts/>  

What is it?
-----------

This quickstart demonstrates how to create a fully **Java EE compliant** project using nothing but  **JBoss Forge**. 

Once generated, the sample project will be a standard Maven 3, Java Web project with **JPA 2.0, EJB 3.1, CDI 1.0, JSF 2.0** with complete **JAX-RS** endpoints for all data Entities. It will also provide views to Create, Read, Update, and Delete records.

But that is not all! You can use Forge on your new or existing projects to continue to enhance any application.

System requirements
-------------------

The application this project produces is designed to be run on Red Hat JBoss Enterprise Application Platform 6.1 or later.

All you need to build this project is Java 6.0 (Java SDK 1.6) or later, and a JBoss Developer Studio 5 or JBoss Forge version 1.0.0.Final (or later).


### JBoss Developer Studio 5

Forge is available in JBoss Developer Studio 5. 

To show the Forge Console, navigate to _Window -> Show View -> Other_, locate _Forge Console_ and click _OK_. Then click the _Start_ button in top right corner of the view.

### JBoss Enterprise Application Platform

If you do not plan to use JBoss Developer Studio, you should install JBoss Forge version 1.0.0.Final or later. Follow the instructions at [Installing Forge](https://docs.jboss.org/author/display/FORGE/Installation).

Open a command line and navigate to the root directory of this quickstart. 

Launch Forge by typing the following command:

        forge
 
 

Generate, Build, and Deploy the Application
-------------------------

In the Forge console, make sure you are in the directory where this README.md file is located:

       forge> cd QUICKSTART_HOME/forge-from-scratch/

Notice that there is a file in this directory named `generate.fsh`. Run this file from Forge using the `run` command:

        forge> run generate.fsh

This command will prompt you to enter a `project-name`, for example: 'example'

After this, you see a lot of console messages with a `[INFO] BUILD SUCCESS` towards the end.


What Did This Create?
-------------------
This quickstart has created a native Java EE 6 application. 

After this command completes, look in your `QUICKSTART_HOME/forge-from-scratch/` folder. You will see a folder with the same name as the `project-name` you entered in the prompt above. Browse through this project to see the code that was generated as a result of this command. 

For a full description of what was generated by running this script and details on the structure of the application, visit the [Forge UI Scaffolding Guide](https://docs.jboss.org/author/display/FORGE/UI+Scaffolding)


Configure Maven 
-------------

If you have not yet done so, you must [Configure Maven](../README.md#configure-maven) before testing the quickstarts. 


Start the JBoss Server
-------------------------

1. Open a command line and navigate to the root of the JBoss server directory.
2. The following shows the command line to start the server:

        For Linux:   JBOSS_HOME/bin/standalone.sh
        For Windows: JBOSS_HOME\bin\standalone.bat


Deploy and test the newly generated application
---------------------------

Make sure you have started the JBoss server. See the instructions in the previous section.

1. Build the application:

        forge> build

2. Use the `jboss-as-7` Forge plugin to deploy the application. Install it with the following command:

        forge> forge install-plugin jboss-as-7

3. Configure the server information:

        forge> as7 setup

    You will be prompted with three questions. Type 'N' for the first two questions to keep the default settings. Enter the path to your server install for the third question.

4. Now deploy the application:

        forge> as7 deploy

    This will deploy `target/YOUR_PROJECT_NAME.war`.
  
5. The application will be running at the following URL: <http://localhost:8080/YOUR_PROJECT_NAME/>.

    Be sure to replace `YOUR_PROJECT_NAME` with the name of the project you chose when running the script.

6. When you are ready to undeploy from JBoss AS, run this command:

        forge> as7 undeploy

 
Next Steps
-------

Open `generate.fsh` and take a look inside! There is not much magic happening here. All of the commands used to generate this project are clearly listed just as if they were typed by your own hands.

Play around with creating more entities, relationships, UI, and generating JAX-RS endpoints,all with just a few simple commands.


Explore plugins! 
----------------

Forge has a rich plugin ecosystem. Want to deploy your application to the Cloud? Use the Forge Openshift plugin: http://github.com/forge/plugin-openshift/

To see a full list of avaialable plugins, make sure that you have an active internet connection and type:

        forge> forge find-plugin *


Run the Quickstart in JBoss Developer Studio or Eclipse
-------------------------------------

You can also start the server and deploy the quickstarts from Eclipse using JBoss tools. For more information, see [Use JBoss Developer Studio or Eclipse to Run the Quickstarts](../README.md#use-jboss-developer-studio-or-eclipse-to-run-the-quickstarts) 


Debug the Application
------------------------------------

If you want to debug the source code or look at the Javadocs of any library in the project, run either of the following commands to pull them into your local repository. The IDE should then detect them.

        mvn dependency:sources
        mvn dependency:resolve -Dclassifier=javadoc
